//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "ADAMEm.h"
#include <vcl/inifiles.hpp>
#include <process.h>

#define Section	"ADAMEm"	// ini file section

static AnsiString &GetShortPath (AnsiString &LongPath)
{
 char buf[1024];
 static AnsiString ShortPath;
 strcpy(buf,LongPath.c_str());
 GetShortPathName(LongPath.c_str(),buf,sizeof(buf));
 ShortPath=AnsiString(buf);
 return ShortPath;
}

static int CheckBounds (int value,int min,int max)
{
 if (value<min) return min;
 if (value>max) return max;
 return value;
}

TADAMEm::TADAMEm()
{
 TempFile[0]='\0';
 SetDefaultOptions();
}

TADAMEm::~TADAMEm()
{
 // Remove temp file if it's still there
 if (TempFile[0]) remove(TempFile);
}

void TADAMEm::SetDefaultOptions(void)
{
 FADAMEmPath="";
 FEOSImage="";
 FWPImage="";
 FOS7Image="";
 FStartupDirectory="";
 FCassetteAImage="";
 FCassetteBImage="";
 FDiskAImage="";
 FDiskBImage="";
 FDiskCImage="";
 FDiskDImage="";
 FCartridgeImage="";
 FVideoMode=0;
 FUPeriod=3;
 FOverscan=1;
 FShowAllSprites=1;
 FSoundDevice=255;
 FVolume=10;
 FSoundQuality=3;
 FPanningLevel=0;
 FReverbLevel=7;
 FChorusLevel=0;
 FSpeakerChannelA=4;
 FSpeakerChannelB=3;
 FSpeakerChannelC=2;
 FSpeakerChannelD=1;
 FSoundLog="";
 FEmuMode=0;
 FIFreq=50;
 FCPUSpeed=100;
 FExpansionModule=0;
 FExpansionModuleDevice=0;
 FSensitivity=200;
 FCalibrate=0;
 FVerbose=1;
 FSync=1;
 FPrinterType=1;
 FJoystick=1;
 FSwapJoystickButtons=0;
 FSwapKeyboardButtons=0;
 FSwapMouseButtons=0;
 FReverseKeypad=0;
 FPrinterLog="";
 FUserOptions="";
 ExpansionRAMPages=1;
 Snapshot="";
 ParallelLog="";
 TDOSVideoMode=0;
 Palette=0;
 AutoSnap=0;
}

void TADAMEm::LoadFromFile(AnsiString &file)
{
 TIniFile &iniFile=*new TIniFile(file);
 ADAMEmPath=iniFile.ReadString(Section,"ADAMEmPath","");
 EOSImage=iniFile.ReadString(Section,"EOSImage","");
 WPImage=iniFile.ReadString(Section,"WPImage","");
 OS7Image=iniFile.ReadString(Section,"OS7Image","");
 StartupDirectory=iniFile.ReadString(Section,"StartupDirectory","");
 CassetteAImage=iniFile.ReadString(Section,"CassetteA","");
 CassetteBImage=iniFile.ReadString(Section,"CassetteB","");
 DiskAImage=iniFile.ReadString(Section,"DiskA","");
 DiskBImage=iniFile.ReadString(Section,"DiskB","");
 DiskCImage=iniFile.ReadString(Section,"DiskC","");
 DiskDImage=iniFile.ReadString(Section,"DiskD","");
 CartridgeImage=iniFile.ReadString(Section,"Cartridge","");
 VideoMode=iniFile.ReadInteger(Section,"VideoMode",0);
 UPeriod=iniFile.ReadInteger(Section,"UPeriod",2);
 Overscan=iniFile.ReadInteger(Section,"Overscan",1);
 ShowAllSprites=iniFile.ReadInteger(Section,"ShowAllSprites",1);
 SoundDevice=iniFile.ReadInteger(Section,"SoundDevice",255);
 Volume=iniFile.ReadInteger(Section,"Volume",10);
 SoundQuality=iniFile.ReadInteger(Section,"SoundQuality",3);
 PanningLevel=iniFile.ReadInteger(Section,"Panning",0);
 ReverbLevel=iniFile.ReadInteger(Section,"Reverb",7);
 ChorusLevel=iniFile.ReadInteger(Section,"Chorus",7);
 SpeakerChannelA=iniFile.ReadInteger(Section,"SpeakerChannelA",1);
 SpeakerChannelB=iniFile.ReadInteger(Section,"SpeakerChannelB",2);
 SpeakerChannelC=iniFile.ReadInteger(Section,"SpeakerChannelC",3);
 SpeakerChannelD=iniFile.ReadInteger(Section,"SpeakerChannelD",4);
 SoundLog=iniFile.ReadString(Section,"SoundLog","");
 EmuMode=iniFile.ReadInteger(Section,"EmuMode",0);
 IFreq=iniFile.ReadInteger(Section,"IFreq",50);
 CPUSpeed=iniFile.ReadInteger(Section,"CPUSpeed",100);
 ExpansionModule=iniFile.ReadInteger(Section,"ExpansionModule",0);
 ExpansionModuleDevice=iniFile.ReadInteger(Section,"ExpansionModuleDevice",0);
 Sensitivity=iniFile.ReadInteger(Section,"Sensitivity",200);
 Calibrate=iniFile.ReadInteger(Section,"Calibrate",0);
 Verbose=iniFile.ReadInteger(Section,"Verbose",1);
 Sync=iniFile.ReadInteger(Section,"Sync",1);
 PrinterType=iniFile.ReadInteger(Section,"PrinterType",1);
 Joystick=iniFile.ReadInteger(Section,"Joystick",1);
 SwapJoystickButtons=iniFile.ReadInteger(Section,"SwapJoystickButtons",0);
 SwapKeyboardButtons=iniFile.ReadInteger(Section,"SwapKeyboardButtons",0);
 SwapMouseButtons=iniFile.ReadInteger(Section,"SwapMouseButtons",0);
 ReverseKeypad=iniFile.ReadInteger(Section,"ReverseKeypad",0);
 PrinterLog=iniFile.ReadString(Section,"PrinterLog","");
 UserOptions=iniFile.ReadString(Section,"UserOptions","");
 ExpansionRAMPages=iniFile.ReadInteger(Section,"ExpansionRAMPages",1);
 Snapshot=iniFile.ReadString(Section,"Snapshot","");
 ParallelLog=iniFile.ReadString(Section,"ParallelLog","");
 TDOSVideoMode=iniFile.ReadInteger(Section,"TDOSVideoMode",0);
 Palette=iniFile.ReadInteger(Section,"Palette",0);
 AutoSnap=iniFile.ReadInteger(Section,"AutoSnap",0);
 delete &iniFile;
}

void TADAMEm::SaveToFile(AnsiString &file)
{
 TIniFile &iniFile=*new TIniFile(file);
 iniFile.WriteString(Section,"ADAMEmPath",ADAMEmPath);
 iniFile.WriteString(Section,"EOSImage",EOSImage);
 iniFile.WriteString(Section,"WPImage",WPImage);
 iniFile.WriteString(Section,"OS7Image",OS7Image);
 iniFile.WriteString(Section,"StartupDirectory",StartupDirectory);
 iniFile.WriteString(Section,"CassetteA",CassetteAImage);
 iniFile.WriteString(Section,"CassetteB",CassetteBImage);
 iniFile.WriteString(Section,"DiskA",DiskAImage);
 iniFile.WriteString(Section,"DiskB",DiskBImage);
 iniFile.WriteString(Section,"DiskC",DiskCImage);
 iniFile.WriteString(Section,"DiskD",DiskDImage);
 iniFile.WriteString(Section,"Cartridge",CartridgeImage);
 iniFile.WriteInteger(Section,"VideoMode",VideoMode);
 iniFile.WriteInteger(Section,"UPeriod",UPeriod);
 iniFile.WriteInteger(Section,"Overscan",Overscan);
 iniFile.WriteInteger(Section,"ShowAllSprites",ShowAllSprites);
 iniFile.WriteInteger(Section,"SoundDevice",SoundDevice);
 iniFile.WriteInteger(Section,"Volume",Volume);
 iniFile.WriteInteger(Section,"SoundQuality",SoundQuality);
 iniFile.WriteInteger(Section,"Panning",PanningLevel);
 iniFile.WriteInteger(Section,"Reverb",ReverbLevel);
 iniFile.WriteInteger(Section,"Chorus",ChorusLevel);
 iniFile.WriteInteger(Section,"SpeakerChannelA",SpeakerChannelA);
 iniFile.WriteInteger(Section,"SpeakerChannelB",SpeakerChannelB);
 iniFile.WriteInteger(Section,"SpeakerChannelC",SpeakerChannelC);
 iniFile.WriteInteger(Section,"SpeakerChannelD",SpeakerChannelD);
 iniFile.WriteString(Section,"SoundLog",SoundLog);
 iniFile.WriteInteger(Section,"EmuMode",EmuMode);
 iniFile.WriteInteger(Section,"IFreq",IFreq);
 iniFile.WriteInteger(Section,"CPUSpeed",CPUSpeed);
 iniFile.WriteInteger(Section,"ExpansionModule",ExpansionModule);
 iniFile.WriteInteger(Section,"ExpansionModuleDevice",ExpansionModuleDevice);
 iniFile.WriteInteger(Section,"Sensitivity",Sensitivity);
 iniFile.WriteInteger(Section,"Calibrate",Calibrate);
 iniFile.WriteInteger(Section,"Verbose",Verbose);
 iniFile.WriteInteger(Section,"Sync",Sync);
 iniFile.WriteInteger(Section,"PrinterType",PrinterType);
 iniFile.WriteInteger(Section,"Joystick",Joystick);
 iniFile.WriteInteger(Section,"SwapJoystickButtons",SwapJoystickButtons);
 iniFile.WriteInteger(Section,"SwapKeyboardButtons",SwapKeyboardButtons);
 iniFile.WriteInteger(Section,"SwapMouseButtons",SwapMouseButtons);
 iniFile.WriteInteger(Section,"ReverseKeypad",ReverseKeypad);
 iniFile.WriteString(Section,"PrinterLog",PrinterLog);
 iniFile.WriteString(Section,"UserOptions",UserOptions);
 iniFile.WriteInteger(Section,"ExpansionRAMPages",ExpansionRAMPages);
 iniFile.WriteString(Section,"Snapshot",Snapshot);
 iniFile.WriteString(Section,"ParallelLog",ParallelLog);
 iniFile.WriteInteger(Section,"TDOSVideoMode",TDOSVideoMode);
 iniFile.WriteInteger(Section,"Palette",Palette);
 iniFile.WriteInteger(Section,"AutoSnap",AutoSnap);
 delete &iniFile;
}

AnsiString &TADAMEm::GetEmulatorOptions (AnsiString &s)
{
 s="";
 if (FEOSImage!="")
  s+="-eos "+GetShortPath(FEOSImage)+"\r\n";
 if (FWPImage!="")
  s+="-wp "+GetShortPath(FWPImage)+"\r\n";
 if (FOS7Image!="")
  s+="-os7 "+GetShortPath(FOS7Image)+"\r\n";
 if (FCassetteAImage!="")
  s+="-tapea "+GetShortPath(FCassetteAImage)+"\r\n";
 if (FCassetteBImage!="")
  s+="-tapeb "+GetShortPath(FCassetteBImage)+"\r\n";
 if (DiskAImage!="")
  s+="-diska "+GetShortPath(FDiskAImage)+"\r\n";
 if (DiskBImage!="")
  s+="-diskb "+GetShortPath(FDiskBImage)+"\r\n";
 if (DiskCImage!="")
  s+="-diskc "+GetShortPath(FDiskCImage)+"\r\n";
 if (DiskDImage!="")
  s+="-diskd "+GetShortPath(FDiskDImage)+"\r\n";
 if (CartridgeImage!="")
  s+=GetShortPath(CartridgeImage)+"\r\n";
 s+="-video "+AnsiString(FVideoMode)+"\r\n";
 s+="-uperiod "+AnsiString(FUPeriod)+"\r\n";
 s+="-overscan "+AnsiString(FOverscan)+"\r\n";
 if (FShowAllSprites) s+="-sprite 0\r\n";
 else s+="-sprite 1\r\n";
 s+="-sound "+AnsiString(FSoundDevice)+"\r\n";
 s+="-volume "+AnsiString(FVolume)+"\r\n";
 s+="-soundquality "+AnsiString(FSoundQuality)+"\r\n";
 s+="-stereo "+AnsiString(FPanningLevel)+"\r\n";
 s+="-reverb "+AnsiString(FReverbLevel)+"\r\n";
 s+="-chorus "+AnsiString(FChorusLevel)+"\r\n";
 s+="-speakerchannels "+AnsiString(FSpeakerChannelA)+","+
 						AnsiString(FSpeakerChannelB)+","+
 						AnsiString(FSpeakerChannelC)+","+
 						AnsiString(FSpeakerChannelD)+"\r\n";
 if (FSoundLog!="")
  s+="-soundtrack "+GetShortPath(FSoundLog)+"\r\n";
 if (FEmuMode==2 ||
 	 (FEmuMode==0 && (FCassetteAImage!="" || FCassetteBImage!="" ||
     				  FDiskAImage!="" || FDiskBImage!="" ||
                      FDiskCImage!="" || FDiskDImage!="" ||
                      CartridgeImage=="")))
  s+="-adam\r\n";
 else
  s+="-cv\r\n";
 s+="-ifreq "+AnsiString(FIFreq)+"\r\n";
 s+="-cpuspeed "+AnsiString(FCPUSpeed)+"\r\n";
 int i;
 switch (FExpansionModule)
 {
  case 1:	// Roller Controller
   i=(FExpansionModuleDevice)? 2:1;
   break;
  case 2:	// Driving Module
   i=(FExpansionModuleDevice)? 4:3;
   break;
  case 3: case 4: case 5:	// Speed Roller. Mouse only
   i=FExpansionModule+2;
   break;
  default:
   i=0;
 }
 s+="-expansion "+AnsiString(i)+"\r\n";
 s+="-sensitivity "+AnsiString(FSensitivity)+"\r\n";
 s+="-calibrate "+AnsiString(FCalibrate)+"\r\n";
 s+="-verbose "+AnsiString(FVerbose)+"\r\n";
 s+="-sync "+AnsiString(FSync)+"\r\n";
 s+="-printertype "+AnsiString(FPrinterType)+"\r\n";
 s+="-joystick "+AnsiString(FJoystick)+"\r\n";
 i=FSwapJoystickButtons | (FSwapKeyboardButtons<<1) | (FSwapMouseButtons<<2);
 s+="-swapbuttons "+AnsiString(i)+"\r\n";
 i=(FReverseKeypad)? 0:1;
 s+="-keypad "+AnsiString(i)+"\r\n";
 if (FPrinterLog!="")
  s+="-printer "+GetShortPath(FPrinterLog)+"\r\n";
 s+="-ram "+AnsiString(ExpansionRAMPages)+"\r\n";
 if (Snapshot!="")
  s+="-snap "+GetShortPath(Snapshot)+"\r\n";
 if (ParallelLog!="")
  s+="-lpt "+GetShortPath(ParallelLog)+"\r\n";
 s+="-tdos "+AnsiString(TDOSVideoMode)+"\r\n";
 s+="-palette "+AnsiString(Palette)+"\r\n";
 s+="-autosnap "+AnsiString(AutoSnap);
 if (FUserOptions!="")
  s+="\r\n"+FUserOptions+"\r\n";
 return s;
}

int TADAMEm::Execute(void)
{
 char buf[MAXPATH+1];
 // If an old temp file still exists, remove it before creating a new one
 if (TempFile[0]) remove(TempFile);
 // Get a temporary file name
 GetTempPath(sizeof(buf),buf);
 GetTempFileName(buf,"m2b",0,TempFile);
 // Get emulator options
 AnsiString ansiString;
 GetEmulatorOptions(ansiString);
 // Open the temp file and put emulator options in it
 FILE *file=fopen(TempFile,"wb");
 if (file)
  if (fwrite(ansiString.c_str(),1,ansiString.Length(),file)!=(size_t)ansiString.Length())
  {
   fclose (file);
   remove (TempFile);
   file=NULL;
  }
 if (!file)
 {
  TempFile[0]='\0';
  return -1;
 }
 fclose (file);
 // Get current directory and drive
 char cwd[MAXPATH];
 getcwd (cwd,sizeof(cwd));
 // Switch to ADAMEm startup directory
 if (FStartupDirectory!="")
 {
  char startupDrive[MAXDRIVE];
  char startupDir[MAXDIR];
  fnsplit (FStartupDirectory.c_str(),startupDrive,startupDir,NULL,NULL);
  setdisk (tolower(startupDrive[0])-'a');
  chdir (startupDir);
 }
 // Spawn ADAMEm. Note the temp file isn't deleted afterwards as we want it to be
 // alive when ADAMEm is initialising. Instead, we delete it at the next call to
 // this function or when the application terminates
 sprintf (buf,"@%s",GetShortPath(AnsiString(TempFile)).c_str());
 int i=spawnl(P_NOWAIT,ADAMEmPath.c_str(),ADAMEmPath.c_str(),buf,NULL);
 // Restore drive and directory
 chdir (cwd);
 setdisk (tolower(cwd[0])-'a');
 return (i==-1)? errno:0;
}

void TADAMEm::SetADAMEmPath (AnsiString &adamEmPath)
{
 FADAMEmPath=adamEmPath;
 char s[MAXPATH];
 char drive[MAXDRIVE];
 char dir[MAXDIR];
 fnsplit (FADAMEmPath.c_str(),drive,dir,NULL,NULL);
 fnmerge (s,drive,dir,"ADAMEm",".txt");
 FReadMeFile=AnsiString(s);
}

void TADAMEm::SetEOSImage (AnsiString &eosImage)
{
 FEOSImage=eosImage;
}

void TADAMEm::SetWPImage (AnsiString &wpImage)
{
 FWPImage=wpImage;
}

void TADAMEm::SetOS7Image (AnsiString &os7Image)
{
 FOS7Image=os7Image;
}

void TADAMEm::SetStartupDirectory (AnsiString &startupDirectory)
{
 FStartupDirectory=startupDirectory;
}

void TADAMEm::SetCassetteAImage (AnsiString &cassetteAImage)
{
 FCassetteAImage=cassetteAImage;
}

void TADAMEm::SetCassetteBImage (AnsiString &cassetteBImage)
{
 FCassetteBImage=cassetteBImage;
}

void TADAMEm::SetDiskAImage (AnsiString &diskAImage)
{
 FDiskAImage=diskAImage;
}

void TADAMEm::SetDiskBImage (AnsiString &diskBImage)
{
 FDiskBImage=diskBImage;
}

void TADAMEm::SetDiskCImage (AnsiString &diskCImage)
{
 FDiskCImage=diskCImage;
}

void TADAMEm::SetDiskDImage (AnsiString &diskDImage)
{
 FDiskDImage=diskDImage;
}

void TADAMEm::SetCartridgeImage (AnsiString &cartridgeImage)
{
 FCartridgeImage=cartridgeImage;
}

void TADAMEm::SetVideoMode (int videoMode)
{
 FVideoMode=CheckBounds(videoMode,0,2);
}

void TADAMEm::SetUPeriod (int uPeriod)
{
 FUPeriod=CheckBounds(uPeriod,1,9);
}

void TADAMEm::SetOverscan (int overscan)
{
 FOverscan=(overscan)? 1:0;
}

void TADAMEm::SetShowAllSprites (int showAllSprites)
{
 FShowAllSprites=(showAllSprites)? 1:0;
}

void TADAMEm::SetSoundDevice (int soundDevice)
{
 if (soundDevice<0) soundDevice=0;
 if (soundDevice>6) soundDevice=255;
 FSoundDevice=soundDevice;
}

void TADAMEm::SetVolume (int volume)
{
 FVolume=CheckBounds (volume,0,15);
}

void TADAMEm::SetSoundQuality (int soundQuality)
{
 FSoundQuality=CheckBounds (soundQuality,1,5);
}

void TADAMEm::SetPanningLevel (int panningLevel)
{
 FPanningLevel=CheckBounds (panningLevel,0,100);
}

void TADAMEm::SetReverbLevel (int reverbLevel)
{
 FReverbLevel=CheckBounds (reverbLevel,0,100);
}

void TADAMEm::SetChorusLevel (int chorusLevel)
{
 FChorusLevel=CheckBounds (chorusLevel,0,100);
}

void TADAMEm::SetSpeakerChannelA (int speakerChannelA)
{
 FSpeakerChannelA=CheckBounds (speakerChannelA,1,4);
}

void TADAMEm::SetSpeakerChannelB (int speakerChannelB)
{
 FSpeakerChannelB=CheckBounds (speakerChannelB,1,4);
}

void TADAMEm::SetSpeakerChannelC (int speakerChannelC)
{
 FSpeakerChannelC=CheckBounds (speakerChannelC,1,4);
}

void TADAMEm::SetSpeakerChannelD (int speakerChannelD)
{
 FSpeakerChannelD=CheckBounds (speakerChannelD,1,4);
}

void TADAMEm::SetSoundLog (AnsiString &soundLog)
{
 FSoundLog=soundLog;
}

void TADAMEm::SetEmuMode (int emuMode)
{
 FEmuMode=CheckBounds (emuMode,0,2);
}

void TADAMEm::SetIFreq (int iFreq)
{
 FIFreq=CheckBounds (iFreq,10,500);
}

void TADAMEm::SetCPUSpeed (int cpuSpeed)
{
 FCPUSpeed=CheckBounds (cpuSpeed,10,999);
}

void TADAMEm::SetExpansionModule (int expansionModule)
{
 FExpansionModule=CheckBounds (expansionModule,0,6);
}

void TADAMEm::SetExpansionModuleDevice (int expansionModuleDevice)
{
 FExpansionModuleDevice=CheckBounds (expansionModuleDevice,0,1);
}

void TADAMEm::SetSensitivity (int sensitivity)
{
 FSensitivity=CheckBounds (sensitivity,1,1000);
}

void TADAMEm::SetCalibrate (int calibrate)
{
 FCalibrate=(calibrate)? 1:0;
}

void TADAMEm::SetVerbose (int verbose)
{
 FVerbose=(verbose)? 1:0;
}

void TADAMEm::SetSync (int sync)
{
 FSync=(sync)? 1:0;
}

void TADAMEm::SetPrinterType (int printerType)
{
 FPrinterType=CheckBounds (printerType,0,3);
}

void TADAMEm::SetJoystick (int joystick)
{
 FJoystick=CheckBounds (joystick,0,3);
}

void TADAMEm::SetSwapJoystickButtons (int swapJoystickButtons)
{
 FSwapJoystickButtons=(swapJoystickButtons)? 1:0;
}

void TADAMEm::SetSwapKeyboardButtons (int swapKeyboardButtons)
{
 FSwapKeyboardButtons=(swapKeyboardButtons)? 1:0;
}

void TADAMEm::SetSwapMouseButtons (int swapMouseButtons)
{
 FSwapMouseButtons=(swapMouseButtons)? 1:0;
}

void TADAMEm::SetReverseKeypad (int reverseKeypad)
{
 FReverseKeypad=(reverseKeypad)? 1:0;
}

void TADAMEm::SetPrinterLog (AnsiString &printerLog)
{
 FPrinterLog=printerLog;
}

void TADAMEm::SetUserOptions (AnsiString &userOptions)
{
 FUserOptions=userOptions;
}

void TADAMEm::SetExpansionRAMPages (int expansionRAMPages)
{
 if (expansionRAMPages<0) expansionRAMPages=0;
 if (expansionRAMPages>255) expansionRAMPages=255;
 FExpansionRAMPages=expansionRAMPages;
}

void TADAMEm::SetSnapshot (AnsiString &snapshot)
{
 FSnapshot=snapshot;
}

void TADAMEm::SetParallelLog (AnsiString &parallelLog)
{
 FParallelLog=parallelLog;
}

void TADAMEm::SetTDOSVideoMode (int tdosVideoMode)
{
 if (tdosVideoMode<0) tdosVideoMode=0;
 if (tdosVideoMode>1) tdosVideoMode=1;
 FTDOSVideoMode=tdosVideoMode;
}

void TADAMEm::SetPalette (int palette)
{
 if (palette<0) palette=0;
 if (palette>1) palette=1;
 FPalette=palette;
}

void TADAMEm::SetAutoSnap (int autoSnap)
{
 FAutoSnap=(autoSnap)? 1:0;
}

